@@ -8,7 +8,7 @@
 #
 ###############################################################################
 #
-#   $Id: Agent.pm 12 2006-09-14 08:50:39Z  $
+#   $Id: Agent.pm 22 2006-09-25 02:14:52Z  $
 #
 #   Description:    This is the base class for all protocol agents. It provides
 #                   the skeletal functionality and management of the LWP::UA
@@ -33,7 +33,8 @@
 #                   set_agent
 #                   _lr_trim
 #
-#   Libraries:      Error
+#   Libraries:      Class::Std
+#                   Error
 #                   LWP::UserAgent
 #                   HTTP::Request
 #                   URI
@@ -56,10 +57,10 @@ use URI;
 use LWP::UserAgent;
 use HTTP::Request;
 
+$VERSION = "0.20";
+
 BEGIN
 {
-    $VERSION = "0.10";
-
     @PROTOS = (COREPROTOS);
     %PROTOS = map { $_ => __PACKAGE__ . "::$_" } @PROTOS;
 }
@@ -85,11 +86,11 @@ my %agent_args : ATTR(:name<agent_args> :default<>);
 ###############################################################################
 sub new
 {
-        my ($class, @argz) = @_;
-        my ($proto, $self);
+    my ($class, @argz) = @_;
+    my ($proto, $self);
 
     # Need to make sure $class is the name, not a reference, for later tests:
-        $class = ref($class) || $class;
+    $class = ref($class) || $class;
 
     # If $class matches this package, then they must specify a protocol
     # as the leading argument (currently only 'REST')
@@ -100,10 +101,10 @@ sub new
             unless $class = $class->class_for_protocol($proto);
         # Make sure it is loaded
         eval "require $class;";
-        $self = $class->new(@argz);
     }
+    my $args = shift(@argz) || {};
 
-    $class->SUPER::new(@argz);
+    return $proto ? $class->new($args) : $class->SUPER::new($args);
 }
 
 ###############################################################################
@@ -188,7 +189,8 @@ sub set_agent
     my ($self, $agent) = @_;
 
     throw Error::Simple("New agent must be derived from LWP::UserAgent")
-        unless (ref($agent) and $agent->isa('LWP::UserAgent'));
+        unless (! defined $agent or
+                (ref($agent) and $agent->isa('LWP::UserAgent')));
 
     $agent{ident $self} = $agent;
 
@@ -222,6 +224,7 @@ sub add_protocol
 {
     my ($class, $proto, $pack) = @_;
 
+    $proto = uc $proto;
     throw Error::Simple("No package specfied for $proto") unless $pack;
 
     push(@PROTOS, $proto);
@@ -555,7 +558,8 @@ sub raw_request : RESTRICTED
 #   Sub Name:       _lr_trim
 #
 #   Description:    Do a right- and left-trim of whitespace and newlines off of
-#                   the passed-in string
+#                   the passed-in string. Also translate newlines and returns
+#                   within a string to spaces, and squeeze sequences of spaces.
 #
 #   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
 #                   $class    in      scalar    Ignored
@@ -568,6 +572,7 @@ sub _lr_trim
 {
     my ($class, $string) = @_;
 
+    $string =~ tr/\n\r\t / /s;
     $string =~ s/^[\s\n]*//;
     $string =~ s/[\s\n]*$//;
 
