@@ -8,7 +8,7 @@
 #
 ###############################################################################
 #
-#   $Id: REST.pm 12 2006-09-14 08:50:39Z  $
+#   $Id: REST.pm 22 2006-09-25 02:14:52Z  $
 #
 #   Description:    This is the protocol-implementation class for making
 #                   requests via the REST interface. At present, this is the
@@ -26,6 +26,7 @@
 #   Libraries:      Class::Std
 #                   Error
 #                   XML::LibXML
+#                   WebService::ISBNDB::Agent
 #
 #   Global Consts:  $VERSION
 #                   $BASEURL
@@ -37,14 +38,21 @@ package WebService::ISBNDB::Agent::REST;
 use 5.6.0;
 use strict;
 use warnings;
-use vars qw($VERSION);
+no warnings 'redefine';
+use vars qw($VERSION $CAN_PARSE_DATES);
 use base 'WebService::ISBNDB::Agent';
 
 use Class::Std;
 use Error;
 use XML::LibXML;
 
-$VERSION = "0.10";
+$VERSION = "0.20";
+
+BEGIN
+{
+    eval "use Date::Parse";
+    $CAN_PARSE_DATES = ($@) ? 0 : 1;
+}
 
 my %baseurl    : ATTR(:name<baseurl>    :default<"http://isbndb.com">);
 my %authors    : ATTR(:name<authors>    :default<"/api/authors.xml">);
@@ -70,6 +78,43 @@ my %parse_table = (
     Subjects   => \&parse_subjects,
 );
 
+###############################################################################
+#
+#   Sub Name:       new
+#
+#   Description:    Pass off to the super-class constructor, which handles
+#                   the special cases for arguments.
+#
+###############################################################################
+sub new
+{
+    shift->SUPER::new(@_);
+}
+
+###############################################################################
+#
+#   Sub Name:       protocol
+#
+#   Description:    Return the name of the protocol we implement; if an
+#                   argument is passed in, test that the argument matches
+#                   our protocol.
+#
+#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
+#                   $self     in      ref       Object
+#                   $test     in      scalar    If passed, test against our
+#                                                 protocol
+#
+#   Returns:        Success:    string or 1
+#                   Failure:    0 if we're testing and the protocol is no match
+#
+###############################################################################
+sub protocol
+{
+    my ($self, $test) = @_;
+
+    return $test ? $test =~ /^rest$/i : 'REST';
+}
+
 ###############################################################################
 #
 #   Sub Name:       request_method
@@ -183,6 +228,8 @@ sub request : RESTRICTED
     throw Error::Simple("XML parse error: $@") if $@;
 
     my $top_elt = $dom->documentElement();
+    throw Error::Simple("Service error: " . $self->_lr_trim($dom->textContent))
+        if (($dom) = $top_elt->getElementsByTagName('ErrorMessage'));
     my ($value, $stats) = $parse_table{$obj->get_type}->($self, $top_elt);
 
     $obj->copy(ref($value) ? $value->[0] : $value) if $overwrite;
@@ -266,7 +313,7 @@ sub parse_authors : RESTRICTED
             $authorref->{categories} = $categories;
         }
         # Look for a list of subjects. We save those in a special format, here.
-        if (($tmp) = $one_author->getElementsByTagName('Subject'))
+        if (($tmp) = $one_author->getElementsByTagName('Subjects'))
         {
             my $subjects = [];
             foreach ($tmp->getElementsByTagName('Subject'))
@@ -381,6 +428,91 @@ sub parse_books : RESTRICTED
 
             $bookref->{authors} = $authors;
         }
+        # Get the Details tag to extract data from the attributes
+        if (($tmp) = $one_book->getElementsByTagName('Details'))
+        {
+            $bookref->{dewey_decimal} = $tmp->getAttribute('dewey_decimal');
+            $bookref->{dewey_decimal_normalized} =
+                $tmp->getAttribute('dewey_decimal_normalized');
+            $bookref->{lcc_number} = $tmp->getAttribute('lcc_number');
+            $bookref->{language} = $tmp->getAttribute('language');
+            $bookref->{physical_description_text} =
+                $tmp->getAttribute('physical_description_text');
+            $bookref->{edition_info} = $tmp->getAttribute('edition_info');
+            $bookref->{change_time} = $tmp->getAttribute('change_time');
+            $bookref->{price_time} = $tmp->getAttribute('price_time');
+            if ($CAN_PARSE_DATES)
+            {
+                $bookref->{change_time_sec} =
+                    str2time($bookref->{change_time}, 'UTC');
+                $bookref->{price_time_sec} =
+                    str2time($bookref->{price_time}, 'UTC');
+            }
+        }
+        # Look for summary text
+        if (($tmp) = $one_book->getElementsByTagName('Summary'))
+        {
+            $bookref->{summary} = $self->_lr_trim($tmp->textContent);
+        }
+        # Look for notes text
+        if (($tmp) = $one_book->getElementsByTagName('Notes'))
+        {
+            $bookref->{notes} = $self->_lr_trim($tmp->textContent);
+        }
+        # Look for URLs text
+        if (($tmp) = $one_book->getElementsByTagName('UrlsText'))
+        {
+            $bookref->{urlstext} = $self->_lr_trim($tmp->textContent);
+        }
+        # Look for awards text
+        if (($tmp) = $one_book->getElementsByTagName('AwardsText'))
+        {
+            $bookref->{awardstext} = $self->_lr_trim($tmp->textContent);
+        }
+        # MARC info block
+        if (($tmp) = $one_book->getElementsByTagName('MARCRecords'))
+        {
+            my $marcs = [];
+            foreach ($tmp->getElementsByTagName('MARC'))
+            {
+                push(@$marcs,
+                     { library_name => $_->getAttribute('library_name'),
+                       last_update  => $_->getAttribute('last_update'),
+                       marc_url     => $_->getAttribute('marc_url') });
+                if ($CAN_PARSE_DATES and $marcs->[$#$marcs]->{last_update})
+                {
+                    $marcs->[$#$marcs]->{last_update_sec} =
+                        str2time($marcs->[$#$marcs]->{last_update}, 'UTC');
+                }
+            }
+            $bookref->{marc} = $marcs;
+        }
+        # Price info block
+        if (($tmp) = $one_book->getElementsByTagName('Prices'))
+        {
+            my $prices = [];
+            foreach ($tmp->getElementsByTagName('Price'))
+            {
+                push(@$prices,
+                     { store_isbn    => $_->getAttribute('store_isbn'),
+                       store_title   => $_->getAttribute('store_title'),
+                       store_url     => $_->getAttribute('store_url'),
+                       store_id      => $_->getAttribute('store_id'),
+                       currency_code => $_->getAttribute('currency_code'),
+                       is_in_stock   => $_->getAttribute('is_in_stock'),
+                       is_historic   => $_->getAttribute('is_historic'),
+                       is_new        => $_->getAttribute('is_new'),
+                       currency_rate => $_->getAttribute('currency_rate'),
+                       price         => $_->getAttribute('price'),
+                       check_time    => $_->getAttribute('check_time') });
+                if ($CAN_PARSE_DATES and $prices->[$#$prices]->{check_time})
+                {
+                    $prices->[$#$prices]->{check_time_sec} =
+                        str2time($prices->[$#$prices]->{check_time}, 'UTC');
+                }
+            }
+            $bookref->{prices} = $prices;
+        }
 
         push(@$books, $class->new($bookref));
     }
@@ -604,7 +736,7 @@ sub parse_subjects : RESTRICTED
         # are all attributes of SubjectData
         $subjectref->{id} = $one_subject->getAttribute('subject_id');
         $subjectref->{book_count} = $one_subject->getAttribute('book_count');
-        $subjectref->{marc_field} = $one_subject->getAttribute('marc_count');
+        $subjectref->{marc_field} = $one_subject->getAttribute('marc_field');
         $subjectref->{marc_indicator_1} =
             $one_subject->getAttribute('marc_indicator_1');
         $subjectref->{marc_indicator_2} =
@@ -703,6 +835,24 @@ to derive the type of data being fetched, and thus the base URI to use. The
 key/value pairs in the hash-reference provided by C<$ARGS> are used in the
 REST protocol to set the query parameters that govern the request.
 
+=item protocol([$TESTVAL])
+
+With no arguments, returns the name of this protocol as a simple string. If
+an argument is passed, it is tested against the protocol name to see if it
+is a match, returning a true or false value as appropriate.
+
+=back
+
+The class also implements a constructor method, which is needed to co-operate
+with the parent class under B<Class::Std> structure. You should generally not
+have to call the constructor directly:
+
+=over 4
+
+=item new([$ARGS])
+
+Calls into the parent constructor with any arguments passed in.
+
 =back
 
 =head1 CAVEATS
